import sys
import re
import datetime
import subprocess
from collections import defaultdict
from datetime import date, timedelta as td, datetime
from PIL import Image
from PIL import ImageFont
from PIL import ImageDraw
from PIL import ImageEnhance

def daterange(start_date, end_date):
    for n in range(int ((end_date - start_date).days)):
        yield start_date + td(n)


command = 'git log --pretty=format:"%ci %h %s" '
pattern = re.compile('^([\d]{4})-([\d]{2})-([\d]{2})')

def generate( repo, prefix ):
    pipe = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, cwd=repo)

    years = {}

    for line in pipe.stdout.readlines():
        match = pattern.search(line.decode('utf-8', 'replace'))
        if match:
            year = int(match.group(1))
            month = int(match.group(2))
            day = int(match.group(3))

            date = datetime(year=year, month=month, day=day)
            if year not in years:
                years[ year ] = {}
            if date not in years[ year ]:
                years[ year ][ date ] = 1
            else:
                years[ year ][ date ] = years[ year ][ date ] + 1

    max = 0
    for year, dates in years.items():
        for date, count in dates.items():
            if count > max:
                max = count

    width = 4
    height = 240

    for year, dates in years.items():

        begin = datetime(year=year, month=1, day=1)
        end = datetime(year=year, month=12, day=31)

        print( "Processing %s - %s" % (prefix, year) )

        img = Image.new("RGBA", (width*366,height), (0,0,0,0))
        draw = ImageDraw.Draw(img)
        index = 0

        for date in daterange(begin, end):
            x1 = index * width
            x2 = x1 + width

            if date.day == 1 and date.month > 1:
                draw.rectangle(((x1,0),(x2,height)), fill="lightgray")

            if date in dates:
                count = dates[ date ]
            else:
                count = 0

            h = (height / max) * count
            color = "orange"

            draw.rectangle(((x1,height-h),(x2,height)), fill=color)

            index = index + 1
        
        img.save("%s-%s.png" % ( prefix, date.year ), "PNG")


if len(sys.argv) > 1:
    repo = str( sys.argv[ 1 ] )
else:
    repo = '.'

if len(sys.argv) > 2:
    prefix = str( sys.argv[ 2 ] )
else:
    prefix = "test"

generate( repo, prefix )
